using System;
using System.Collections.Generic;
using Engine.Physics2D;
using Syntriax.Engine.Core;
using Syntriax.Engine.Core.Abstract;
using Syntriax.Engine.Physics2D.Abstract;
using Syntriax.Engine.Physics2D.Primitives;

namespace Syntriax.Engine.Physics2D;

public class Collider2DAABBBehaviour : BehaviourOverride, ICollider2D
{
    public AABB AABBLocal { get; set; } = null!;
    public AABB AABBWorld { get; private set; } = null!;

    private IRigidBody2D? _rigidBody2D = null;
    private List<Vector2D> _vertices = new List<Vector2D>(4);

    public IRigidBody2D? RigidBody2D
    {
        get
        {
            if (_rigidBody2D is null)
                BehaviourController.TryGetBehaviour(out _rigidBody2D);

            return _rigidBody2D;
        }
    }

    public Action<ICollider2D, ICollider2D>? OnCollisionPreResolve { get; set; } = null;

    public Action<IAssignableTransform>? OnTransformAssigned { get => GameObject.OnTransformAssigned; set => GameObject.OnTransformAssigned = value; }
    ITransform IAssignableTransform.Transform => Transform;
    public bool Assign(ITransform transform) => GameObject.Assign(transform);

    public IList<Vector2D> Vertices => _vertices;

    public bool CheckCollision(Vector2D point)
    {
        return AABBWorld.Overlaps(point);
    }

    public void Recalculate()
    {
        AABBWorld = new AABB(
            AABBLocal.LowerBoundary.Scale(Transform.Scale) + Transform.Position,
            AABBLocal.UpperBoundary.Scale(Transform.Scale) + Transform.Position
        );

        Vertices.Clear();
        Vertices.Add(AABBWorld.LowerBoundary);
        Vertices.Add(new Vector2D(AABBWorld.LowerBoundary.X, AABBWorld.UpperBoundary.Y));
        Vertices.Add(AABBWorld.UpperBoundary);
        Vertices.Add(new Vector2D(AABBWorld.UpperBoundary.X, AABBWorld.LowerBoundary.Y));
    }
    public Collider2DAABBBehaviour(Vector2D lowerBoundary, Vector2D upperBoundary)
    {
        AABBLocal = new AABB(lowerBoundary, upperBoundary);
        AABBWorld = new AABB(lowerBoundary, upperBoundary);
    }

    public Collider2DAABBBehaviour()
    {
        AABBLocal = new(Vector2D.Zero, Vector2D.Zero);
        AABBWorld = new(Vector2D.Zero, Vector2D.Zero);
    }
}
